@@ -153,8 +153,8 @@ class BoundingBox(object):
             self._xmax = float(centroid[0]) + float(width) / 2.0
             self._ymax = float(centroid[1]) + float(height) / 2.0
         elif xcentroid is not None and ycentroid is not None and width is not None and height is not None:
-            if not (isnumber(xcentroid) and isnumber(ycentroid) and isnumber(width) and isnumber(height)):
-                raise ValueError('Box coordinates must be integers or floats')
+            #if not (isnumber(xcentroid) and isnumber(ycentroid) and isnumber(width) and isnumber(height)):
+            #    raise ValueError('Box coordinates must be integers or floats')
             self._xmin = float(xcentroid) - (float(width) / 2.0)
             self._ymin = float(ycentroid) - (float(height) / 2.0)
             self._xmax = float(xcentroid) + (float(width) / 2.0)
@@ -214,11 +214,68 @@ class BoundingBox(object):
 
     def __str__(self):
         return self.__repr__()
-
+    
     def xmin(self):
         """x coordinate of upper left corner of box, x-axis is image column"""
         return self._xmin
 
+    def ul(self):
+        """Upper left coordinate (x,y)"""
+        return (self._xmin, self._ymin)
+
+    def ulx(self):
+        """Upper left coordinate (x)"""
+        return self.ul()[0]
+
+    def uly(self):
+        """Upper left coordinate (y)"""
+        return self.ul()[1]
+
+    def ur(self):
+        """Upper right coordinate (x,y)"""
+        return (self._xmax, self._ymin)
+
+    def urx(self):
+        """Upper right coordinate (x)"""
+        return self.ur()[0]
+
+    def ury(self):
+        """Upper right coordinate (y)"""
+        return self.ur()[1]
+
+    def ll(self):
+        """Lower left coordinate (x,y), synonym for bl()"""
+        return (self._xmin, self._ymax)
+
+    def bl(self):
+        """Bottom left coordinate (x,y), synonym for ll()"""
+        return (self._xmin, self._ymax)
+
+    def blx(self):
+        """Bottom left coordinate (x)"""
+        return self.bl()[0]
+
+    def bly(self):
+        """Bottom left coordinate (y)"""
+        return self.bl()[1]
+
+    def lr(self):
+        """Lower right coordinate (x,y), synonym for br()"""
+        return (self._xmax, self._ymax)
+
+    def br(self):
+        """Bottom right coordinate (x,y), synonym for lr()"""
+        return (self._xmax, self._ymax)
+
+    def brx(self):
+        """Bottom right coordinate (x)"""
+        return self.br()[0]
+
+    def bry(self):
+        """Bottom right coordinate (y)"""
+        return self.br()[1]
+
+
     def ymin(self):
         """y coordinate of upper left corner of box, y-axis is image row"""
         return self._ymin
@@ -247,6 +304,12 @@ class BoundingBox(object):
         """Return the (x,y) lower right corner coordinate of the box"""
         return (self.xmax(), self.ymax())
 
+    def isinteger(self):
+        return (isinstance(self._xmin, int) and
+                isinstance(self._ymin, int) and
+                isinstance(self._xmax, int) and
+                isinstance(self._ymax, int))
+                
     def int(self):
         """Convert corners to integer with rounding"""
         (w,h) = (int(np.round(self.bbwidth())), int(np.round(self.bbheight())))
@@ -285,6 +348,14 @@ class BoundingBox(object):
         self._ymax = self._ymax + dy
         return self
 
+    def to_origin(self):
+        return self.translate(-self.xmin(), -self.ymin())
+    
+    def set_origin(self, other):
+        """Set the origin of the coordinates of this bounding box to be relative to the upper left of the other bounding box"""
+        assert isinstance(other, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(other))
+        return self.translate(other.xmin(), other.ymin())                
+    
     def offset(self, dx=0, dy=0):
         """Alias for translate"""
         return self.translate(dx, dy)
@@ -406,7 +477,7 @@ class BoundingBox(object):
     def ulbr(self, ulbr=None):
         """Return bounding box corners as upper left, bottom right (xmin, ymin, xmax, ymax)"""
         if ulbr is None:
-            return (self.xmin(), self.ymin(), self.xmax(), self.ymax())            
+            return (self._xmin, self._ymin, self._xmax, self._ymax)            
         else:
             assert len(ulbr) == 4, "Invalid (xmin,ymin,xmax,ymax) input"
             self._xmin = float(ulbr[0])
@@ -444,18 +515,19 @@ class BoundingBox(object):
 
     def iou(self, bb):
         """area of intersection / area of union"""
-        assert isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))        
-        if bb is None or bb.invalid() or self.invalid():
-            return 0.0
-        w = min(self.xmax(), bb.xmax()) - max(self.xmin(), bb.xmin())
-        h = min(self.ymax(), bb.ymax()) - max(self.ymin(), bb.ymin())
-        if ((w < 0) or (h < 0)):
-            iou = 0   # invalid (no overlap)
-        else:
-            area_intersection = w * h
-            area_union = (self.area() + bb.area() - area_intersection)
-            iou = area_intersection / float(area_union)
-        return iou
+        assert bb is None or isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))        
+        if bb is None:
+            return 0
+        w = min(self._xmax, bb._xmax) - max(self._xmin, bb._xmin)
+        if w <= 0:
+            return 0  # invalid (no overlap), early exit
+        h = min(self._ymax, bb._ymax) - max(self._ymin, bb._ymin)
+        if h <= 0:
+            return 0  # invalid (no overlap), early exit
+
+        area_intersection = w * h
+        area_union = (self.area() + bb.area() - area_intersection)
+        return area_intersection / float(area_union)
 
     def intersection_over_union(self, bb):
         """Alias for iou"""
@@ -464,57 +536,68 @@ class BoundingBox(object):
     def area_of_intersection(self, bb):
         """area of intersection"""
         assert isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))                
-        if bb.invalid() or self.invalid():
-            return 0.0
-        w = min(self.xmax(), bb.xmax()) - max(self.xmin(), bb.xmin())
-        h = min(self.ymax(), bb.ymax()) - max(self.ymin(), bb.ymin())
-        if ((w < 0) or (h < 0)):
-            aoi = 0   # invalid (no overlap)
-        else:
-            aoi = w * h
-        return aoi
+        w = min(self._xmax, bb._xmax) - max(self._xmin, bb._xmin)
+        if w <= 0:
+            return 0  # invalid (no overlap), early exit 
+        h = min(self._ymax, bb._ymax) - max(self._ymin, bb._ymin)
+        if h <= 0:
+            return 0  # invalid (no overlap), early exit 
+        return w*h
 
     def cover(self, bb):
-        """Fraction of this bounding box intersected by other bbox"""        
+        """Fraction of this bounding box intersected by other bbox (bb)"""        
         return self.area_of_intersection(bb) / float(self.area())
 
+    def maxcover(self, bb):
+        """The maximum cover of self to bb and bb to self"""
+        return max(self.cover(bb), bb.cover(self))
+    
     def shapeiou(self, bb):
         """Shape IoU is the IoU with the upper left corners aligned. This measures the deformation of the two boxes by removing the effect of translation"""
+        #return self.iou(bb.clone().translate(dx=self._xmin-bb._xmin, dy=self._ymin-bb._ymin))  # equivalent to
         assert isinstance(bb, BoundingBox), "Invalid input - must be BoundingBox()"
-        return self.iou(bb.clone().translate(dx=self.xmin()-bb.xmin(), dy=self.ymin()-bb.ymin()))
-
+        w = min(self._xmax, bb._xmax + (self._xmin-bb._xmin)) - max(self._xmin, bb._xmin + (self._xmin-bb._xmin))
+        h = min(self._ymax, bb._ymax + (self._ymin-bb._ymin)) - max(self._ymin, bb._ymin + (self._ymin-bb._ymin))
+        area_intersection = w * h
+        area_union = (self.area() + bb.area() - area_intersection)
+        return area_intersection / float(area_union)
+        
     def intersection(self, bb, strict=True):
         """Intersection of two bounding boxes, throw an error on degeneracy of intersection result (if strict=True)"""
         assert isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))                        
-        self._xmin = max(bb.xmin(), self.xmin())
-        self._ymin = max(bb.ymin(), self.ymin())
-        self._xmax = min(bb.xmax(), self.xmax())
-        self._ymax = min(bb.ymax(), self.ymax())
+        self._xmin = max(bb._xmin, self._xmin)
+        self._ymin = max(bb._ymin, self._ymin)
+        self._xmax = min(bb._xmax, self._xmax)
+        self._ymax = min(bb._ymax, self._ymax)
         if strict and self.isdegenerate():
             raise ValueError('Degenerate intersection for bounding boxes "%s" and "%s"' % (str(bb), str(self)))
         return self
 
     def hasintersection(self, bb):
-        """Return true of self and bb intersect"""
-        return self.area_of_intersection(bb) > 0
+        """Return true of self and bb intersect, do not perform type check to maximize speed"""
+        #assert isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))
+        return (((self._xmax if self._xmax < bb._xmax else bb._xmax) - (self._xmin if self._xmin > bb._xmin else bb._xmin)) > 0 and
+                ((self._ymax if self._ymax < bb._ymax else bb._ymax) - (self._ymin if self._ymin > bb._ymin else bb._ymin)) > 0)
 
     def union(self, bb):
         """Union of one or more bounding boxes with this box"""        
         bblist = tolist(bb)        
         assert all([isinstance(bb, BoundingBox) for bb in bblist]), "Invalid BoundingBox() input"
-        self._xmin = min([bb.xmin() for bb in bblist] + [self.xmin()])
-        self._ymin = min([bb.ymin() for bb in bblist] + [self.ymin()])
-        self._xmax = max([bb.xmax() for bb in bblist] + [self.xmax()])
-        self._ymax = max([bb.ymax() for bb in bblist] + [self.ymax()])
+        self._xmin = min([bb._xmin for bb in bblist] + [self._xmin])
+        self._ymin = min([bb._ymin for bb in bblist] + [self._ymin])
+        self._xmax = max([bb._xmax for bb in bblist] + [self._xmax])
+        self._ymax = max([bb._ymax for bb in bblist] + [self._ymax])
         return self
 
-    def inside(self, p):
+    def isinside(self, bb):
+        """Is this boundingbox fully within the provided bounding box?"""
+        assert isinstance(bb, BoundingBox)
+        return self.hasintersection(bb) and self.cover(bb) == 1.0
+        
+    def ispointinside(self, p):
         """Is the 2D point p=(x,y) inside this boundingbox, or is the p=boundingbox() inside this bounding box?"""
-        if isinstance(p, BoundingBox):
-            return self.hasintersection(p) and self.cover(p) == 1.0
-        else:
-            assert len(p) == 2, "Invalid 2D point=(x,y) input"""
-            return (p[0] >= self._xmin) and (p[1] >= self._ymin) and (p[0] <= self._xmax) and (p[1] <= self._ymax)
+        assert len(p) == 2, "Invalid 2D point=(x,y) input"""
+        return (p[0] >= self._xmin) and (p[1] >= self._ymin) and (p[0] <= self._xmax) and (p[1] <= self._ymax)
 
     def dilate(self, scale=1):
         """Change scale of bounding box keeping centroid constant"""
@@ -532,6 +615,14 @@ class BoundingBox(object):
         self._ymax = c[1] + new_y
         return self
 
+    def dilatepx(self, px):
+        """Dilate by a given pixel amount on all sides, keeping centroid constant"""
+        self._xmin = self._xmin - px
+        self._ymin = self._ymin - px
+        self._xmax = self._xmax + px
+        self._ymax = self._ymax + px
+        return self
+
     def dilate_height(self, scale=1):
         """Change scale of bounding box in y direction keeping centroid constant"""
         h = self.height()
@@ -696,6 +787,15 @@ class BoundingBox(object):
             assert isnumber(width) and isnumber(height), "Invalid width and height - both must be numbers"
         return self.area_of_intersection(BoundingBox(xmin=0, ymin=0, width=width, height=height)) > 0
 
+    def isinterior(self, width=None, height=None, border=1.0):
+        """Is this boundingbox fully within the provided image rectangle?  
+        
+           * If border in [0,1], then the image is dilated by a border percentage prior to computing interior, useful to check if self is near the image edge
+           * If border=0.8, then the image rectangle is dilated by 80% (smaller) keeping the centroid constant. 
+        """
+        assert border > 0 and border <= 1, "Border must be a dilation fraction of the image, such that the image centroid is constant and the sides are dilated by a scale [0,1]"
+        return self.isinside(imagebox((height, width)).dilate(border))
+
     def iminterior(self, W, H):
         """Transform bounding box to be interior to the image rectangle with shape (W,H).  
            Transform is applyed by computing smallest (dx,dy) translation that it is interior to the image rectangle, then clip to the image rectangle if it is too big to fit
@@ -757,7 +857,7 @@ class BoundingBox(object):
         return Ellipse(self.bbwidth() / 2.0, self.bbheight() / 2.0, xcenter, ycenter, 0)
 
     def average(self, other):
-        """Compute the average bounding box between self and other.  Other may be a singleton bounding box or a list of bounding boxes"""
+        """Compute the average bounding box between self and other, and set self to the average.  Other may be a singleton bounding box or a list of bounding boxes"""
         assert all([isinstance(bb, BoundingBox) for bb in tolist(other)]), "Invalid input - must be BoundingBox"        
         return self.ulbr(np.mean( [self.ulbr()] + [bb.ulbr() for bb in tolist(other)], axis=0))
 
@@ -793,15 +893,20 @@ class BoundingBox(object):
         return self.centroid(dehomogenize(np.dot(A, homogenize(np.array(self.centroid())))))
     
     def crop(self, img):
-        """Crop an HxW 2D numpy image, HxWxC 3D numpy image, or NxHxWxC 4D numpy image array using this bounding box applied to HxW dimensions.  Sets bounding box to integer coordinates"""
+        """Crop an HxW 2D numpy image, HxWxC 3D numpy image, or NxHxWxC 4D numpy image array using this bounding box applied to HxW dimensions.  Crop is performed in-place. """
         assert isnumpy(img) and img.ndim in [2,3,4]
+        assert self.isinteger(), "Box corners must be integer - try calling self.int()"
+
         if img.ndim == 2:
-            return img[self.int().ymin():self.ymax(), self.xmin():self.xmax()]  # HxW
+            return img[self.ymin():self.ymax(), self.xmin():self.xmax()]  # HxW
         elif img.ndim == 3:
             return img[self.ymin():self.ymax(), self.xmin():self.xmax(), :]  # HxWxC
         else: 
             return img[:, self.ymin():self.ymax(), self.xmin():self.xmax(), :]  # NxHxWxC
 
+    def confidence(self):
+        """Bounding boxes do not have confidences, use vipy.object.Detection()"""
+        return None
 
 class Ellipse():
     __slots__ = ['_major', '_minor', '_xcenter', '_ycenter', '_phi']
